#include "irensei_rule_description.h"

#include <string.h>


#define FIRST_VERSION	99
#define LATEST_VERSION	100



#ifdef __cplusplus
extern "C" {
#endif


IRENSEI_RULE_DESCRIPTION * MakeDefaultIrenseiRuleDescriptionVer99( void * buf )
{
	return MakeDefaultIrenseiRuleDescriptionVer100( buf );
}

IRENSEI_RULE_DESCRIPTION * MakeDefaultIrenseiRuleDescriptionVer100( void * buf )
{
	IRENSEI_RULE_DESCRIPTION * rule = (IRENSEI_RULE_DESCRIPTION *)buf;

	strcpy( rule->signature, "IRDESC!" );
	rule->version = 100;			// W[o[W * 100 + }Ci[o[W
	rule->lineLength = 7;			// 7
	rule->requiredLines = 1;		// ɕKvȘA̐
	rule->outerInvalidLines = 2;	// VAɐȂHBOA̘H܂ł͘AɐȂ

	return rule;
}


static int check_rule_99( const IRENSEI_RULE_DESCRIPTION * rule )
{
	if ( 0 != strcmp( rule->signature, "IRDESC!" ) )
		return 0;
	if ( rule->version != 99 )
		return 0;
	if ( rule->lineLength < 4 || rule->lineLength > 13 )
		return 0;
	if ( rule->requiredLines != 1 )
		return 0;
	if ( rule->outerInvalidLines < 0 || rule->outerInvalidLines > 7 )
		return 0;

	return 1;
}

static int check_rule_100( const IRENSEI_RULE_DESCRIPTION * rule )
{
	if ( 0 != strcmp( rule->signature, "IRDESC!" ) )
		return 0;
	if ( rule->version != 100 )
		return 0;
	if ( rule->lineLength != 7 )
		return 0;
	if ( rule->requiredLines != 1 )
		return 0;
	if ( rule->outerInvalidLines != 2 )
		return 0;

	return 1;
}


typedef struct {
	int version;
	int (*checker)( const IRENSEI_RULE_DESCRIPTION *  );
} RuleChecker;


static RuleChecker rule_checkers[] = {
	{ 99, check_rule_99 },
	{ 100, check_rule_100 },
};



int CheckIrenseiRuleDescriptionValidity( const IRENSEI_RULE_DESCRIPTION * rule )
{
	if ( rule == 0 ) {
		return LATEST_VERSION;
	}

	if ( rule->version < FIRST_VERSION || rule->version > LATEST_VERSION ) {
		return 0;
	}

	// o[WƂ̔胋[`Ń`FbN
	{
		const RuleChecker * checker = rule_checkers;
		const RuleChecker * end = &rule_checkers[ sizeof(rule_checkers)/sizeof(*rule_checkers) ];
		for ( ; checker < end; checker++ ) {
			if ( checker->version == rule->version ) {
				return (*checker->checker)(rule);
			}
		}
	}

	return 0;
}




#ifdef __cplusplus
}
#endif


